"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Router_1 = require("../Router");
const Server_1 = require("../Server");
const Controller_1 = require("../Controller");
const Response_1 = require("../Response");
const ModuleManager_1 = require("../ModuleManager");
const defaultConfig = __importStar(require("../../config"));
const debug_1 = require("../../debug");
const Config_1 = require("../Config");
const ServiceManager_1 = require("../ServiceManager");
const EventManager_1 = require("../EventManager");
const ApplicationEvents_1 = require("./ApplicationEvents");
const LoggerService_1 = require("../Logger/LoggerService");
const debug = debug_1.createDebugLogger('application');
class Application {
    constructor(appConfig) {
        this.config = new Config_1.Config(defaultConfig, appConfig);
        this.serviceManager = new ServiceManager_1.ServiceManager({
            aliases: { config: Config_1.Config, sharedEventManager: EventManager_1.SharedEventManager },
            invokables: new Map([
                [EventManager_1.SharedEventManager, EventManager_1.SharedEventManager],
            ]),
            factories: new Map([
                [ModuleManager_1.ModuleManager, ModuleManager_1.ModuleManagerFactory],
                [EventManager_1.EventManager, EventManager_1.EventManagerFactory],
            ]),
            services: new Map([
                [Config_1.Config, this.config],
                [Application, this],
            ]),
            shared: new Map([
                [EventManager_1.EventManager, false],
            ]),
        });
    }
    bootstrap() {
        return __awaiter(this, void 0, void 0, function* () {
            const config = this.config;
            // Make the module manager. Only one level is allowed to specify module configs..
            this.moduleManager = this.serviceManager.get(ModuleManager_1.ModuleManager);
            // Initialize module manager. Only calls getConfig()
            yield this.moduleManager.loadModules(config.of('modules'));
            // Now that we have all the configs, register the services.
            this.serviceManager.configure(config.of('services'));
            // go forth and create all core services.
            this.logger = this.serviceManager.get(LoggerService_1.LoggerService);
            this.router = this.serviceManager.get(Router_1.RouterService);
            this.server = this.serviceManager.get(Server_1.ServerService);
            this.responseService = this.serviceManager.get(Response_1.ResponseService);
            this.controllerManager = this.serviceManager.get(Controller_1.ControllerManager);
            this.sharedEventManager = this.serviceManager.get(EventManager_1.SharedEventManager);
            // Now it's our turn. Modules had their chance, now we get to register our default middleware.
            yield this.server.initialize(config.of('server'));
            // Cool cool. Bootstrap the modules, because they can now get all the things.
            yield this.moduleManager.bootstrap();
            // Allow listeners to do some work before starting the server.
            yield this.sharedEventManager.trigger(ApplicationEvents_1.ApplicationEvents.Ready, this);
            return this;
        });
    }
    getLogger() {
        return this.logger;
    }
    getRouter() {
        return this.router;
    }
    getServer() {
        return this.server;
    }
    getControllerManager() {
        return this.controllerManager;
    }
    getModuleManager() {
        return this.moduleManager;
    }
    getResponseService() {
        return this.responseService;
    }
    getServiceManager() {
        return this.serviceManager;
    }
    getConfigOf(section) {
        return this.config.of(section);
    }
    getConfig() {
        return this.config;
    }
    launch() {
        return __awaiter(this, void 0, void 0, function* () {
            debug('Launching server');
            yield this.bootstrap();
            this.server.start();
            debug('Server ready');
            return this;
        });
    }
}
exports.Application = Application;
//# sourceMappingURL=Application.js.map