"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const ServiceManager_1 = require("../ServiceManager");
class ControllerManager extends ServiceManager_1.AbstractPluginManager {
    constructor(creationContext, config) {
        super(creationContext, config.controllers);
        if (config.locations) {
            this.loadFromLocations(config.locations);
        }
    }
    static getControllerName(controller) {
        if (typeof controller === 'string') {
            return controller;
        }
        return controller.name;
    }
    loadFromLocations(controllerDirectories) {
        controllerDirectories.forEach(directory => this.loadDirectory(directory));
        return this;
    }
    loadDirectory(controllerDirectory) {
        const controllers = fs_1.default.readdirSync(controllerDirectory)
            .filter((fileName) => !!fileName.match(/^(?!(index)).*[^d]\.(js|ts)$/))
            .map((fileName) => fileName.replace(/\.(js|ts)$/, ''))
            .map((controller) => {
            const Controller = require(path_1.default.resolve(controllerDirectory, controller));
            if (typeof Controller === 'function') {
                return Controller;
            }
            if (typeof Controller.default === 'function') {
                return Controller.default;
            }
            if (typeof Controller[controller] === 'function') {
                return Controller[controller];
            }
            throw new TypeError(`Unable to load controller "${controller}" due to missing constructable export.`);
        });
        this.registerControllers(controllers);
    }
    getController(controller) {
        return this.get(controller);
    }
    registerControllers(controllers) {
        controllers.forEach(Controller => this.registerController(Controller));
        return this;
    }
    registerController(Controller) {
        this.registerInvokable(Controller, Controller);
        this.registerAlias(Controller.name, Controller);
        return this;
    }
}
exports.ControllerManager = ControllerManager;
//# sourceMappingURL=ControllerManager.js.map