"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ModuleManagerEvents_1 = require("./ModuleManagerEvents");
const debug_1 = require("../../debug");
const debug = debug_1.createDebugLogger('modules');
class ModuleManager {
    constructor(application, eventManager, config) {
        this.application = application;
        this.eventManager = eventManager;
        this.config = config;
    }
    bootstrap() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.eventManager.trigger(ModuleManagerEvents_1.ModuleManagerEvents.OnBootstrap, this);
        });
    }
    loadModule(ModuleClass) {
        return __awaiter(this, void 0, void 0, function* () {
            debug('Loading module ' + ModuleClass.name);
            const eventManager = this.eventManager;
            const module = new ModuleClass();
            if (typeof module.getConfig === 'function') {
                this.config.merge(yield module.getConfig());
            }
            if (typeof module.init === 'function') {
                module.init(this);
            }
            // Allow for a convenience onBootstrap method.
            if (typeof module.onBootstrap === 'function' && !eventManager.has(ModuleManagerEvents_1.ModuleManagerEvents.OnBootstrap, module.onBootstrap)) {
                debug(`Auto-attaching onBootstrap listener for ${ModuleClass.name}.`);
                eventManager.attachOnce(ModuleManagerEvents_1.ModuleManagerEvents.OnBootstrap, module.onBootstrap);
            }
            debug('Initialized module ' + ModuleClass.name);
            return this;
        });
    }
    getEventManager() {
        return this.eventManager;
    }
    getApplication() {
        return this.application;
    }
    loadModules(config) {
        return __awaiter(this, void 0, void 0, function* () {
            debug('Loading modules');
            for (let i = 0; i < config.length; i++) {
                yield this.loadModule(config[i]);
            }
            debug('Loaded modules');
            return this;
        });
    }
}
exports.ModuleManager = ModuleManager;
//# sourceMappingURL=ModuleManager.js.map