"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const koa_1 = __importDefault(require("koa"));
const cors_1 = __importDefault(require("@koa/cors"));
const koa_bodyparser_1 = __importDefault(require("koa-bodyparser"));
const middleware_1 = require("../../middleware");
class ServerService {
    constructor(application) {
        this.application = application;
        this.server = new koa_1.default();
    }
    use(...middlewares) {
        middlewares.forEach(middleware => this.server.use(middleware));
        return this;
    }
    useBefore(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware), 0, ...middlewares);
    }
    useAfter(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware) + 1, 0, ...middlewares);
    }
    replace(middleware, ...middlewares) {
        return this.updateMiddleware(this.indexOfMiddleware(middleware), 1, ...middlewares);
    }
    updateMiddleware(at, remove, ...middlewares) {
        const middleware = this.server.middleware;
        if (at === -1) {
            throw new Error();
        }
        middleware.splice(at, remove, ...middlewares);
        return this;
    }
    indexOfMiddleware(middleware) {
        if (typeof middleware === 'function') {
            return this.server.middleware.indexOf(middleware);
        }
        return this.server.middleware.findIndex(suspect => suspect.name === middleware);
    }
    initialize(config) {
        return __awaiter(this, void 0, void 0, function* () {
            this.config = config;
            if (config.cors.enabled) {
                this.use(cors_1.default(config.cors.options));
            }
            this.use(middleware_1.requestMiddleware(), koa_bodyparser_1.default(), middleware_1.routerMiddleware(this.application), middleware_1.dispatchMiddleware(this.application));
        });
    }
    getServer() {
        return this.server;
    }
    start() {
        this.server.listen(this.config.port);
        return this;
    }
}
exports.ServerService = ServerService;
//# sourceMappingURL=ServerService.js.map