"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Library_1 = require("../Library");
const Server_1 = require("../Library/Server");
const debug_1 = require("../debug");
const debug = debug_1.createDebugLogger('middleware:router');
exports.routerMiddleware = (app) => function router(ctx, next) {
    return __awaiter(this, void 0, void 0, function* () {
        debug(`Routing for path "${ctx.path}".`);
        const method = ctx.method === Server_1.RequestMethods.Head ? Server_1.RequestMethods.Get : ctx.method;
        const match = app.getRouter().resolve(method, ctx.path);
        if (!match) {
            ctx.state.response = app.getResponseService().clientError().notFound();
            debug('No match, returning not found.');
            return;
        }
        const { route, parameters } = match;
        const controllerName = Library_1.ControllerManager.getControllerName(route.controller);
        const controller = app.getControllerManager().get(route.controller);
        ctx.state.params = parameters;
        ctx.state.dispatch = {
            controllerName,
            controller,
            action: route.action,
        };
        debug(`Route matched "${controllerName}.${route.action}()".`);
        return next();
    });
};
//# sourceMappingURL=routerMiddleware.js.map